import pygame
import xml.dom.minidom
import os

'''


'''

class Axis:
    LEFT_X = 0
    LEFT_Y = 1
    RIGHT_X = 2
    RIGHT_Y = 3

class TwinSticks():
    def __init__( self ):

        pygame.init()
        pygame.joystick.init() # main joystick device system

        self.rightX = -1
        self.rightXFlip = 1

        self.rightY = -1
        self.rightYFlip = 1


        self.leftX = -1
        self.leftXFlip = 1

        self.leftY = -1
        self.leftYFlip = 1

        self.controller = None
        try:
        	self.controller = pygame.joystick.Joystick(0) # create a joystick instance
        	self.controller.init() # init instance
        	self.name = self.controller.get_name()
        except pygame.error:
        	self.controller = None

        # get all the axis so as we calibrate, the target axis can be removed
        # to not trigger the next calibration
        self.axes = range( self.controller.get_numaxes() )

        self.calibrationLoaded = False
        filename = "data/gamepadCalibration.xml"
        if os.path.isfile( filename ):
            controllerDom = xml.dom.minidom.parse( filename )
            for ctrlNode in controllerDom.getElementsByTagName( "controller" ):
                if ctrlNode.getAttribute("name") == self.name:
                    for mapping in ctrlNode.getElementsByTagName( "axisMapping" ):
                        if mapping.getAttribute("name") == "leftX":
                            self.leftX = int(mapping.getAttribute("axisIndex" ))
                            self.leftXFlip = int(mapping.getAttribute("axisFlip" ))

                        elif mapping.getAttribute("name") == "leftY":
                            self.leftY = int(mapping.getAttribute("axisIndex" ))
                            self.leftYFlip = int(mapping.getAttribute("axisFlip" ))

                        elif mapping.getAttribute("name") == "rightX":
                            self.rightX = int(mapping.getAttribute("axisIndex" ))
                            self.rightXFlip = int(mapping.getAttribute("axisFlip" ))

                        elif mapping.getAttribute("name") == "rightY":
                            self.rightY = int(mapping.getAttribute("axisIndex" ))
                            self.rightYFlip = int(mapping.getAttribute("axisFlip" ))
                    self.calibrationLoaded = True
                    break
            controllerDom.unlink()


    def Calibrate( self, axis ):
        for i in self.axes:
            axisVal = self.controller.get_axis( i )

            if abs( axisVal ) > 0.8:
                if axis == Axis.LEFT_X:
                    print "RADCITY"
                    self.leftX = i
                    if axisVal > 0:
                        self.leftXFlip = -1
                    self.axes.remove( i )
                    return True

                elif axis == Axis.LEFT_Y:
                    self.leftY = i
                    if axisVal > 0:
                        self.leftYFlip = -1
                    self.axes.remove( i )
                    return True

                elif axis == Axis.RIGHT_X:
                    self.rightX = i
                    if axisVal > 0:
                        self.rightXFlip = -1
                    self.axes.remove( i )
                    return True

                elif axis == Axis.RIGHT_Y:
                    self.rightY = i
                    if axisVal > 0:
                        self.rightYFlip = -1
                    self.axes.remove( i )
                    return True

        return False

    def GetLeftX( self ):
        return self.controller.get_axis( self.leftX ) * self.leftXFlip

    def GetLeftY( self ):
        return self.controller.get_axis( self.leftY ) * self.leftYFlip

    def GetRightX( self ):
        return self.controller.get_axis( self.rightX ) * self.rightXFlip

    def GetRightY( self ):
        return self.controller.get_axis( self.rightY ) * self.rightYFlip


    def _Mapping( self, dom, axisName, axisIndex, axisFlip ):

        mapping = dom.createElement( "axisMapping" )
        mapping.setAttribute("name", axisName )
        mapping.setAttribute("axisIndex", str(axisIndex) )
        mapping.setAttribute("axisFlip", str(axisFlip) )
        return mapping

    def _AddController( self, dom ):
        newController = dom.createElement( "controller" )
        newController.setAttribute( "name", self.name )

        newController.appendChild( self._Mapping( dom, "leftX", self.leftX, self.leftXFlip ) )
        newController.appendChild( self._Mapping( dom, "leftY", self.leftY, self.leftYFlip ) )
        newController.appendChild( self._Mapping( dom, "rightX", self.rightX, self.rightXFlip ) )
        newController.appendChild( self._Mapping( dom, "rightY", self.rightY, self.rightYFlip ) )

        return newController

    def SaveCalibration( self ):
        filename = "data/gamepadCalibration.xml"

        found = False
        if os.path.isfile( filename ):
            controllerDom = xml.dom.minidom.parse( filename )
            for ctrlNode in controllerDom.documentElement.childNodes:
                if ctrlNode.getAttribute("name") == self.name:
                    found = True
                    for mapping in ctrlNode.childNodes:
                        if mapping.getAttribute("name") == "leftX":
                            mapping.setAttribute("axisIndex", self.leftX)
                            mapping.setAttribute("axisFlip", self.leftXFlip)

                        elif mapping.getAttribute("name") == "leftY":
                            mapping.setAttribute("axisIndex", self.leftY)
                            mapping.setAttribute("axisFlip", self.leftYFlip)

                        elif mapping.getAttribute("name") == "leftX":
                            mapping.setAttribute("axisIndex", self.leftX)
                            mapping.setAttribute("axisFlip", self.leftXFlip)

                        elif mapping.getAttribute("name") == "rightY":
                            mapping.setAttribute("axisIndex", self.rightY)
                            mapping.setAttribute("axisFlip", self.rightYFlip)

            if not found:
                controllerDom.appendChild( self._AddController( self, controllerDom ) )
        else:
            newDoc = xml.dom.minidom.getDOMImplementation()
            controllerDom = newDoc.createDocument( None, "controllers", None)

            controllerDom.documentElement.appendChild( self._AddController( controllerDom ) )

        fid = open( filename, "w" )
        fid.write( controllerDom.toprettyxml() )
        fid.close()

        controllerDom.unlink()